use DE_EPAGES::Core::API::File qw ( CopyFile DeleteFile GetFileSize RunOnDirectory );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::EPagesJ::API::PDF::PdfClient;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use File::Basename;
use Test::More;

use strict;

sub TestPdf {
    my ($Pdf) = @_;
    return $Pdf =~ /^%PDF/ ? 1 : 0;
}

sub testPdfTransform {
    my $System = LoadRootObject();
    my $Client = DE_EPAGES::EPagesJ::API::PDF::PdfClient->create( $System, { isLogging => 1 } );

    my $HtmlCode = '<html><head></head><body><p>blubb</p></body></html>';
    my $Pdf      = $Client->transform($HtmlCode);
    ok( TestPdf($Pdf), 'valid xhtml' );

    $HtmlCode = '<html><head></head><body><p>blubb</body></html>';
    $Pdf      = $Client->transform($HtmlCode);
    ok( TestPdf($Pdf), 'missing end tag' );

    $HtmlCode = '<html><head></head><body att=sddf><p>blubb</p></body></html>';
    $Pdf      = $Client->transform($HtmlCode);
    ok( TestPdf($Pdf), 'missing quotes' );

    $HtmlCode = '<html><head></head><body att=sddf"><p>blubb</p></body></html>';
    $Pdf      = $Client->transform($HtmlCode);
    ok( TestPdf($Pdf), 'odd number of qoutes (left one is missing)' );

    $HtmlCode = '<html><head></head><body att="sddf><p>blubb</p></body></html>';
    $Pdf      = $Client->transform($HtmlCode);
    ok( TestPdf($Pdf), 'odd number of qoutes (right one is missing)' );

    $HtmlCode = '<html><head></head><body att="sddf"><p>blubb</p></body>';
    $Pdf      = $Client->transform($HtmlCode);
    ok( TestPdf($Pdf), '</html> is missing' );

    $HtmlCode = '<head></head><body att="sddf"><p>blubb</p></body></html>';
    $Pdf      = $Client->transform($HtmlCode);
    ok( TestPdf($Pdf), '<html> is missing' );

    $HtmlCode = '<p>blubb</p>';
    $Pdf      = $Client->transform($HtmlCode);
    ok( TestPdf($Pdf), 'valid xhtml snippet' );

    $HtmlCode = '<p>blubb';
    $Pdf      = $Client->transform($HtmlCode);
    ok( TestPdf($Pdf), 'missing end tag at xhtml snippet' );
}

sub testPdfMerge {
    my $System = LoadRootObject();
    my $Client = DE_EPAGES::EPagesJ::API::PDF::PdfClient->create( $System, { isLogging => 1 } );

    # copy source pdf
    my $StaticPath = 'Static/' . GetTestSite() . '/PdfClient.t/fileContent1.pdf';
    CopyFile( 'fileContent1.pdf', $ENV{'EPAGES_SHARED'} . '/' . $StaticPath, { create_directory => 1 } );

    my $Pdf = $Client->merge( { 'files' => [ { 'path' => $StaticPath } ], } );

    # enable for debugging the output pdf.
    # WriteFile('testFile.pdf', \$Pdf, {binary=>1});
    ok( TestPdf($Pdf), 'Merged PDF header check' );
    ok( abs( length($Pdf) - GetFileSize('fileContent1.pdf') ) < 100, 'fuzzy check for file size of merged PDF' );

    # cleanup
    DeleteFile( $ENV{'EPAGES_SHARED'} . '/' . $StaticPath );
}

sub Main {
    RunOnDirectory(
        dirname(__FILE__),
        sub {
            RunOnStore(
                'Store' => GetTestSite(),
                'Sub'   => sub {

                    # tests
                    testPdfTransform();
                    testPdfMerge();
                    done_testing();
                }
            );
        }
    );
}

RunScript( Sub => \&Main, Debug => 1 );

1;
